@@ -1,3 +1,5 @@
+import { IFontManager } from '@deathbeds/jupyterlab-fonts';
+import { GlobalStyles } from '@deathbeds/jupyterlab-fonts/lib/_schema';
 import { LabShell } from '@jupyterlab/application';
 import { ISettingRegistry } from '@jupyterlab/settingregistry';
 import { StatusBar } from '@jupyterlab/statusbar';
@@ -5,10 +7,9 @@ import { TranslationBundle } from '@jupyterlab/translation';
 import { each } from '@lumino/algorithm';
 import { CommandRegistry } from '@lumino/commands';
 import { Signal, ISignal } from '@lumino/signaling';
-import { Widget, DockPanel, BoxLayout } from '@lumino/widgets';
+import { Widget, DockPanel } from '@lumino/widgets';
 
 import { ICONS } from './icons';
-import { DeckRemote } from './remote';
 import {
   IDeckManager,
   DATA,
@@ -22,9 +23,38 @@ import {
   DIRECTION_KEYS,
   CSS,
   COMPOUND_KEYS,
+  IStylePreset,
+  IDeckSettings,
+  TSlideType,
+  TLayerScope,
 } from './tokens';
+import { DesignTools } from './tools/design';
+import type { Layover } from './tools/layover';
+import { DeckRemote } from './tools/remote';
 
 export class DeckManager implements IDeckManager {
+  protected _active = false;
+  protected _activeChanged = new Signal<IDeckManager, void>(this);
+  protected _activeWidget: Widget | null = null;
+  protected _presenters: IPresenter<any>[] = [];
+  protected _appStarted: Promise<void>;
+  protected _commands: CommandRegistry;
+  protected _remote: DeckRemote | null = null;
+  protected _designTools: DesignTools | null = null;
+  protected _settings: Promise<ISettingRegistry.ISettings>;
+  protected _shell: LabShell;
+  protected _statusbar: StatusBar | null;
+  protected _statusBarWasEnabled = false;
+  protected _styleCache = new Map<HTMLElement, string>();
+  protected _trans: TranslationBundle;
+  protected _stylePresets = new Map<string, IStylePreset>();
+  protected _stylePresetsChanged = new Signal<IDeckManager, void>(this);
+  protected _layoverChanged = new Signal<IDeckManager, void>(this);
+  protected _fonts: IFontManager;
+  protected _layover: Layover | null = null;
+  protected _activePresenter: IPresenter<Widget> | null = null;
+  protected _activeWidgetStack: Widget[] = [];
+
   constructor(options: DeckManager.IOptions) {
     this._appStarted = options.appStarted;
     this._commands = options.commands;
@@ -32,11 +62,12 @@ export class DeckManager implements IDeckManager {
     this._statusbar = options.statusbar;
     this._trans = options.translator;
     this._settings = options.settings;
+    this._fonts = options.fonts;
 
     this._shell.activeChanged.connect(this._onActiveWidgetChanged, this);
     this._shell.layoutModified.connect(this._addDeckStylesLater, this);
-    this._registerCommands();
-    this._registerKeyBindings();
+    this._addCommands();
+    this._addKeyBindings();
     this._settings
       .then(async (settings) => {
         settings.changed.connect(this._onSettingsChanged, this);
@@ -45,10 +76,30 @@ export class DeckManager implements IDeckManager {
       .catch(console.warn);
   }
 
+  public get activePresenter() {
+    return this._activePresenter;
+  }
+
+  public get layover() {
+    return this._layover;
+  }
+
+  public get fonts() {
+    return this._fonts;
+  }
+
   public get activeChanged(): ISignal<IDeckManager, void> {
     return this._activeChanged;
   }
 
+  public get stylePresetsChanged(): ISignal<IDeckManager, void> {
+    return this._stylePresetsChanged;
+  }
+
+  public get layoverChanged(): ISignal<IDeckManager, void> {
+    return this._layoverChanged;
+  }
+
   /**
    * translate a string by message id (usually the en-US string), potentially
    * with positional arguments, starting with %1.
@@ -64,12 +115,25 @@ export class DeckManager implements IDeckManager {
     presenter.activeChanged.connect(() => this._activeChanged.emit(void 0));
   }
 
+  public addStylePreset(preset: IStylePreset): void {
+    this._stylePresets.set(preset.key, preset);
+    this._stylePresetsChanged.emit(void 0);
+  }
+
+  public get stylePresets(): IStylePreset[] {
+    return [...this._stylePresets.values()];
+  }
+
   /** enable deck mode */
-  public start = async (): Promise<void> => {
+  public start = async (force: boolean = false): Promise<void> => {
     await this._appStarted;
-    if (this._active) {
+    const wasActive = this._active;
+
+    /* istanbul ignore if */
+    if (wasActive && !force) {
       return;
     }
+
     if (!this._activeWidget) {
       const { _shellActiveWidget } = this;
       if (_shellActiveWidget) {
@@ -79,27 +143,35 @@ export class DeckManager implements IDeckManager {
         return;
       }
     }
+
     const { _shell, _activeWidget } = this;
-    this._active = true;
-    void this._settings.then((settings) => settings.set('active', true));
-    if (this._statusbar) {
-      this._statusBarWasEnabled = this._statusbar.isVisible;
-      this._statusbar.hide();
+
+    if (!wasActive) {
+      this._active = true;
+      void this._settings.then((settings) => settings.set('active', true));
+      if (this._statusbar) {
+        this._statusBarWasEnabled = this._statusbar.isVisible;
+        this._statusbar.hide();
+      }
+      _shell.presentationMode = false;
+      document.body.dataset[DATA.deckMode] = DATA.presenting;
+      each(this._dockpanel.tabBars(), (bar) => bar.hide());
+      _shell.mode = 'single-document';
+      this._remote = new DeckRemote({ manager: this });
+      this._designTools = new DesignTools({ manager: this });
+      window.addEventListener('resize', this._addDeckStylesLater);
     }
-    _shell.presentationMode = false;
-    document.body.dataset[DATA.deckMode] = DATA.presenting;
-    each(this._dockpanel.tabBars(), (bar) => bar.hide());
-    _shell.mode = 'single-document';
     _shell.update();
-    this._remote = new DeckRemote({ manager: this });
-    (_shell.layout as BoxLayout).addWidget(this._remote);
-    window.addEventListener('resize', this._addDeckStylesLater);
+
     await this._onActiveWidgetChanged();
 
     if (_activeWidget) {
       const presenter = this._getPresenter(_activeWidget);
       if (presenter) {
+        this._activePresenter = presenter;
         await presenter.start(_activeWidget);
+      } else {
+        this._activePresenter = null;
       }
     }
 
@@ -111,8 +183,11 @@ export class DeckManager implements IDeckManager {
       _shell.collapseLeft();
       _shell.collapseRight();
     }, 1000);
+
+    if (!wasActive) {
+      this._addDeckStyles();
+    }
     this._activeChanged.emit(void 0);
-    this._addDeckStyles();
   };
 
   public get activeWidget(): Widget | null {
@@ -121,11 +196,17 @@ export class DeckManager implements IDeckManager {
 
   /** disable deck mode */
   public stop = async (): Promise<void> => {
+    /* istanbul ignore if */
     if (!this._active) {
       return;
     }
 
-    const { _activeWidget, _shell, _statusbar, _remote } = this;
+    const { _activeWidget, _shell, _statusbar, _remote, _layover, _designTools } = this;
+
+    /* istanbul ignore if */
+    if (_layover) {
+      await this.hideLayover();
+    }
 
     if (_activeWidget) {
       const presenter = this._getPresenter(_activeWidget);
@@ -144,21 +225,28 @@ export class DeckManager implements IDeckManager {
       _remote.dispose();
       this._remote = null;
     }
+    if (_designTools) {
+      _designTools.dispose();
+      this._designTools = null;
+    }
     _shell.presentationMode = false;
     _shell.mode = 'multiple-document';
     window.removeEventListener('resize', this._addDeckStylesLater);
     delete document.body.dataset[DATA.deckMode];
     this._activeWidget = null;
     this._active = false;
+    this._activeWidgetStack = [];
     void this._settings.then((settings) => settings.set('active', false));
   };
 
   /** move around */
   public go = async (direction: TDirection, alternate?: TDirection): Promise<void> => {
+    /* istanbul ignore if */
     if (!this._activeWidget) {
       return;
     }
     const presenter = this._getPresenter(this._activeWidget);
+    /* istanbul ignore if */
     if (!presenter) {
       return;
     }
@@ -189,11 +277,12 @@ export class DeckManager implements IDeckManager {
         }
       }
     }
+    /* istanbul ignore next */
     return null;
   }
 
   /** overload the stock notebook keyboard shortcuts */
-  protected _registerKeyBindings() {
+  protected _addKeyBindings() {
     for (const direction of Object.values(DIRECTION)) {
       this._commands.addKeyBinding({
         command: CommandIds[direction],
@@ -213,12 +302,84 @@ export class DeckManager implements IDeckManager {
     }
   }
 
-  protected _registerCommands() {
+  public async showLayover() {
+    if (!this._layover) {
+      this._layover = new (await import('./tools/layover')).Layover({ manager: this });
+      this._layoverChanged.emit(void 0);
+    }
+    await this.start(true);
+  }
+
+  public async hideLayover() {
+    if (this._layover) {
+      this._layover.dispose();
+      this._layover = null;
+      this._layoverChanged.emit(void 0);
+    }
+  }
+
+  public getSlideType(): TSlideType {
+    let { _activeWidget, _activePresenter } = this;
+    if (_activeWidget && _activePresenter?.getSlideType) {
+      return _activePresenter.getSlideType(_activeWidget) || null;
+    }
+    /* istanbul ignore next */
+    return null;
+  }
+
+  public setSlideType(slideType: TSlideType): void {
+    let { _activeWidget, _activePresenter } = this;
+    if (_activeWidget && _activePresenter?.setSlideType) {
+      _activePresenter.setSlideType(_activeWidget, slideType);
+    }
+  }
+
+  public getLayerScope(): TLayerScope | null {
+    let { _activeWidget, _activePresenter } = this;
+    if (_activeWidget && _activePresenter?.getLayerScope) {
+      return _activePresenter.getLayerScope(_activeWidget) || null;
+    }
+    /* istanbul ignore next */
+    return null;
+  }
+
+  public setLayerScope(layerScope: TLayerScope | null): void {
+    let { _activeWidget, _activePresenter } = this;
+    if (_activeWidget && _activePresenter?.setLayerScope) {
+      _activePresenter.setLayerScope(_activeWidget, layerScope);
+    }
+  }
+
+  public getPartStyles(): GlobalStyles | null {
+    let { _activeWidget, _activePresenter } = this;
+    if (_activeWidget && _activePresenter?.getPartStyles) {
+      const styles = _activePresenter.getPartStyles(_activeWidget) || null;
+      return styles;
+    }
+    /* istanbul ignore next */
+    return null;
+  }
+  public setPartStyles(styles: GlobalStyles | null): void {
+    let { _activeWidget, _activePresenter } = this;
+    if (_activeWidget && _activePresenter?.setPartStyles) {
+      _activePresenter.setPartStyles(_activeWidget, styles);
+    }
+  }
+
+  public activateWidget(widget: Widget): void {
+    this._shell.activateById(widget.node.id);
+  }
+
+  public get activeWidgetStack(): Widget[] {
+    return [...this._activeWidgetStack];
+  }
+
+  protected _addCommands() {
     let { _commands, __, go } = this;
     _commands.addCommand(CommandIds.start, {
       label: __('Start Deck'),
       icon: ICONS.deckStart,
-      execute: this.start,
+      execute: () => this.start(),
     });
     _commands.addCommand(CommandIds.stop, {
       label: __('Stop Deck'),
@@ -232,6 +393,19 @@ export class DeckManager implements IDeckManager {
         await (this._active ? this.stop() : this.start());
       },
     });
+
+    this._commands.addCommand(CommandIds.showLayover, {
+      icon: ICONS.transformStart,
+      label: this.__('Show Slide Layout'),
+      execute: () => this.showLayover(),
+    });
+
+    this._commands.addCommand(CommandIds.hideLayover, {
+      icon: ICONS.transformStop,
+      label: this.__('Hide Slide Layout'),
+      execute: () => this.hideLayover(),
+    });
+
     _commands.addCommand(CommandIds.go, {
       label: __('Go direction in Deck'),
       execute: async (args: any) => {
@@ -258,13 +432,14 @@ export class DeckManager implements IDeckManager {
 
   /** handle the active widget changing */
   protected async _onActiveWidgetChanged(): Promise<void> {
+    if (!this._active) {
+      this._activeWidget = null;
+      return;
+    }
+
     const { _activeWidget, _shellActiveWidget } = this;
 
-    if (
-      !_shellActiveWidget ||
-      _shellActiveWidget === _activeWidget ||
-      _shellActiveWidget === this._remote
-    ) {
+    if (!_shellActiveWidget || _shellActiveWidget === _activeWidget) {
       /* modals and stuff? */
       return;
     }
@@ -274,21 +449,33 @@ export class DeckManager implements IDeckManager {
       if (presenter) {
         await presenter.stop(_activeWidget);
       }
+      if (!this._activeWidgetStack.includes(_activeWidget)) {
+        this._activeWidgetStack.push(_activeWidget);
+      }
     }
 
     this._activeWidget = _shellActiveWidget;
 
-    if (this._active) {
-      if (_shellActiveWidget) {
-        const presenter = this._getPresenter(_shellActiveWidget);
-        if (presenter) {
-          await presenter.start(_shellActiveWidget);
-        }
+    if (_shellActiveWidget) {
+      if (this._activeWidgetStack.includes(_shellActiveWidget)) {
+        this._activeWidgetStack.splice(
+          this._activeWidgetStack.indexOf(_shellActiveWidget),
+          1
+        );
       }
-
-      this._addDeckStyles();
-      this._activeChanged.emit(void 0);
+      const presenter = this._getPresenter(_shellActiveWidget);
+      if (presenter) {
+        this._activePresenter = presenter;
+        await presenter.start(_shellActiveWidget);
+      } else {
+        this._activePresenter = null;
+      }
+    } else {
+      this._activePresenter = null;
     }
+
+    this._addDeckStyles();
+    this._activeChanged.emit(void 0);
   }
 
   protected get _shellActiveWidget(): Widget | null {
@@ -302,27 +489,50 @@ export class DeckManager implements IDeckManager {
 
   protected async _onSettingsChanged() {
     const settings = await this._settings;
-    const { composite } = settings;
-    const active = composite['active'] === true;
+    let composite: IDeckSettings;
+    composite = settings.composite as IDeckSettings;
+    const active = composite.active === true;
+
     if (active && !this._active) {
       void this.start();
     } else if (!active && this._active) {
       void this.stop();
     }
-  }
 
-  cacheStyle(node: HTMLElement) {
-    if (!this._styleCache.get(node)) {
-      this._styleCache.set(node, node.getAttribute('style') || '');
+    if (composite.stylePresets) {
+      for (let keyPreset of Object.entries(composite.stylePresets)) {
+        let [key, preset] = keyPreset;
+        let { scope, label, styles } = preset;
+        if (!styles || !label) {
+          continue;
+        }
+        this._stylePresets.set(key, {
+          key,
+          scope: scope || 'any',
+          styles,
+          label,
+        });
+        this._stylePresetsChanged.emit(void 0);
+      }
     }
-    node.setAttribute('style', '');
   }
 
-  uncacheStyle(node: HTMLElement) {
-    const style = this._styleCache.get(node);
-    if (style) {
-      node.setAttribute('style', style);
-      this._styleCache.delete(node);
+  cacheStyle(...nodes: HTMLElement[]) {
+    for (const node of nodes) {
+      if (!this._styleCache.get(node)) {
+        this._styleCache.set(node, node.getAttribute('style') || '');
+      }
+      node.setAttribute('style', '');
+    }
+  }
+
+  uncacheStyle(...nodes: HTMLElement[]) {
+    for (const node of nodes) {
+      const style = this._styleCache.get(node);
+      if (style) {
+        node.setAttribute('style', style);
+        this._styleCache.delete(node);
+      }
     }
   }
 
@@ -353,20 +563,6 @@ export class DeckManager implements IDeckManager {
     this._shell.fit();
     setTimeout(this._addDeckStyles, 10);
   };
-
-  protected _active = false;
-  protected _activeChanged = new Signal<IDeckManager, void>(this);
-  protected _activeWidget: Widget | null = null;
-  protected _presenters: IPresenter<any>[] = [];
-  protected _appStarted: Promise<void>;
-  protected _commands: CommandRegistry;
-  protected _remote: DeckRemote | null = null;
-  protected _settings: Promise<ISettingRegistry.ISettings>;
-  protected _shell: LabShell;
-  protected _statusbar: StatusBar | null;
-  protected _statusBarWasEnabled = false;
-  protected _styleCache = new Map<HTMLElement, string>();
-  protected _trans: TranslationBundle;
 }
 
 export namespace DeckManager {
@@ -377,6 +573,7 @@ export namespace DeckManager {
     statusbar: StatusBar | null;
     settings: Promise<ISettingRegistry.ISettings>;
     appStarted: Promise<void>;
+    fonts: IFontManager;
   }
   export interface IExtent {
     onScreen: number[];
