@@ -6,13 +6,19 @@ import { IDisposable, DisposableDelegate } from '@lumino/disposable';
 
 import { CommandIds } from '../tokens';
 
+import { NotebookPresenter } from './presenter';
+
 export class NotebookDeckExtension
   implements DocumentRegistry.IWidgetExtension<NotebookPanel, INotebookModel>
 {
   private _commands: CommandRegistry;
+  private _presenter: NotebookPresenter;
+
   constructor(options: DeckExtension.IOptions) {
     this._commands = options.commands;
+    this._presenter = options.presenter;
   }
+
   createNew(
     panel: NotebookPanel,
     context: DocumentRegistry.IContext<INotebookModel>
@@ -24,6 +30,9 @@ export class NotebookDeckExtension
     });
 
     panel.toolbar.insertItem(5, 'deck', button);
+
+    this._presenter.preparePanel(panel);
+
     return new DisposableDelegate(() => button.dispose());
   }
 }
@@ -31,5 +40,6 @@ export class NotebookDeckExtension
 export namespace DeckExtension {
   export interface IOptions {
     commands: CommandRegistry;
+    presenter: NotebookPresenter;
   }
 }
